@@ -1,7 +1,7 @@
 //! Implements Dijkstra's algorithm for shortest path, to find an
 //! optimal and readable diff between two ASTs.
 
-use std::cmp::Reverse;
+use std::{cmp::Reverse, env};
 
 use crate::{
     graph::{mark_route, neighbours, Edge, Vertex},
@@ -25,7 +25,9 @@ fn shortest_path(start: Vertex) -> Vec<(Edge, Vertex)> {
     // usage.
     let mut predecessors: FxHashMap<Vertex, PredecessorInfo> = FxHashMap::default();
 
-    let mut neighbour_buf = [None, None, None, None, None, None, None, None, None, None];
+    let mut neighbour_buf = [
+        None, None, None, None, None, None, None, None, None, None, None, None,
+    ];
     let end = loop {
         match heap.pop() {
             Some((Reverse(distance), current)) => {
@@ -73,18 +75,21 @@ fn shortest_path(start: Vertex) -> Vec<(Edge, Vertex)> {
     route.reverse();
 
     info!("Found a path of {} with cost {}.", route.len(), cost);
+    let print_length = if env::var("DFT_VERBOSE").is_ok() {
+        50
+    } else {
+        5
+    };
     info!(
-        "Initial path: {:#?}",
-        route.iter().map(|x| x.0).take(5).collect_vec()
+        "Initial {} items on path: {:#?}",
+        print_length,
+        route.iter().map(|x| x.0).take(print_length).collect_vec()
     );
     route
 }
 
 pub fn mark_syntax<'a>(lhs_syntax: Option<&'a Syntax<'a>>, rhs_syntax: Option<&'a Syntax<'a>>) {
-    let start = Vertex {
-        lhs_syntax,
-        rhs_syntax,
-    };
+    let start = Vertex::new(lhs_syntax, rhs_syntax);
     let route = shortest_path(start);
     mark_route(&route);
 }
@@ -126,19 +131,11 @@ mod tests {
         let rhs = Syntax::new_atom(&arena, pos_helper(0), "foo", AtomKind::Normal);
         init_all_info(&[lhs], &[rhs]);
 
-        let start = Vertex {
-            lhs_syntax: Some(lhs),
-            rhs_syntax: Some(rhs),
-        };
+        let start = Vertex::new(Some(lhs), Some(rhs));
         let route = shortest_path(start);
 
         let actions = route.iter().map(|(action, _)| *action).collect_vec();
-        assert_eq!(
-            actions,
-            vec![UnchangedNode {
-                depth_difference: 0
-            }]
-        );
+        assert_eq!(actions, vec![UnchangedNode]);
     }
 
     #[test]
@@ -169,20 +166,16 @@ mod tests {
         )];
         init_all_info(&lhs, &rhs);
 
-        let start = Vertex {
-            lhs_syntax: lhs.get(0).copied(),
-            rhs_syntax: rhs.get(0).copied(),
-        };
+        let start = Vertex::new(lhs.get(0).copied(), rhs.get(0).copied());
         let route = shortest_path(start);
 
         let actions = route.iter().map(|(action, _)| *action).collect_vec();
         assert_eq!(
             actions,
             vec![
-                UnchangedDelimiter {
-                    depth_difference: 0
-                },
-                NovelAtomLHS { contiguous: false }
+                EnterUnchangedDelimiter,
+                NovelAtomLHS { contiguous: false },
+                ExitDelimiterBoth,
             ]
         );
     }
@@ -213,21 +206,17 @@ mod tests {
         )];
         init_all_info(&lhs, &rhs);
 
-        let start = Vertex {
-            lhs_syntax: lhs.get(0).copied(),
-            rhs_syntax: rhs.get(0).copied(),
-        };
+        let start = Vertex::new(lhs.get(0).copied(), rhs.get(0).copied());
         let route = shortest_path(start);
 
         let actions = route.iter().map(|(action, _)| *action).collect_vec();
         assert_eq!(
             actions,
             vec![
-                UnchangedDelimiter {
-                    depth_difference: 0
-                },
+                EnterUnchangedDelimiter,
                 NovelAtomRHS { contiguous: false },
-                NovelAtomRHS { contiguous: false }
+                NovelAtomRHS { contiguous: false },
+                ExitDelimiterBoth,
             ]
         );
     }
@@ -261,24 +250,19 @@ mod tests {
         )];
         init_all_info(&lhs, &rhs);
 
-        let start = Vertex {
-            lhs_syntax: lhs.get(0).copied(),
-            rhs_syntax: rhs.get(0).copied(),
-        };
+        let start = Vertex::new(lhs.get(0).copied(), rhs.get(0).copied());
         let route = shortest_path(start);
 
         let actions = route.iter().map(|(action, _)| *action).collect_vec();
         assert_eq!(
             actions,
             vec![
-                NovelDelimiterRHS { contiguous: false },
-                NovelDelimiterLHS { contiguous: false },
-                UnchangedNode {
-                    depth_difference: 0
-                },
-                UnchangedNode {
-                    depth_difference: 0
-                }
+                EnterNovelDelimiterRHS { contiguous: false },
+                EnterNovelDelimiterLHS { contiguous: false },
+                UnchangedNode,
+                UnchangedNode,
+                ExitDelimiterLHS,
+                ExitDelimiterRHS,
             ],
         );
     }
@@ -301,19 +285,14 @@ mod tests {
         )];
         init_all_info(&lhs, &rhs);
 
-        let start = Vertex {
-            lhs_syntax: lhs.get(0).copied(),
-            rhs_syntax: rhs.get(0).copied(),
-        };
+        let start = Vertex::new(lhs.get(0).copied(), rhs.get(0).copied());
         let route = shortest_path(start);
 
         let actions = route.iter().map(|(action, _)| *action).collect_vec();
         assert_eq!(
             actions,
             vec![
-                UnchangedNode {
-                    depth_difference: 0
-                },
+                UnchangedNode,
                 NovelAtomLHS { contiguous: false },
                 NovelAtomLHS { contiguous: true },
             ]
@@ -341,18 +320,16 @@ mod tests {
         let rhs = vec![];
         init_all_info(&lhs, &rhs);
 
-        let start = Vertex {
-            lhs_syntax: lhs.get(0).copied(),
-            rhs_syntax: rhs.get(0).copied(),
-        };
+        let start = Vertex::new(lhs.get(0).copied(), rhs.get(0).copied());
         let route = shortest_path(start);
 
         let actions = route.iter().map(|(action, _)| *action).collect_vec();
         assert_eq!(
             actions,
             vec![
-                NovelDelimiterLHS { contiguous: false },
+                EnterNovelDelimiterLHS { contiguous: false },
                 NovelAtomLHS { contiguous: true },
+                ExitDelimiterLHS,
             ]
         );
     }
@@ -381,19 +358,17 @@ mod tests {
         let rhs = vec![];
         init_all_info(&lhs, &rhs);
 
-        let start = Vertex {
-            lhs_syntax: lhs.get(0).copied(),
-            rhs_syntax: rhs.get(0).copied(),
-        };
+        let start = Vertex::new(lhs.get(0).copied(), rhs.get(0).copied());
         let route = shortest_path(start);
 
         let actions = route.iter().map(|(action, _)| *action).collect_vec();
         assert_eq!(
             actions,
             vec![
-                NovelDelimiterLHS { contiguous: false },
+                EnterNovelDelimiterLHS { contiguous: false },
+                NovelAtomLHS { contiguous: true },
+                ExitDelimiterLHS,
                 NovelAtomLHS { contiguous: true },
-                NovelAtomLHS { contiguous: true }
             ]
         );
     }
@@ -466,10 +441,7 @@ mod tests {
         )];
         init_all_info(&lhs, &rhs);
 
-        let start = Vertex {
-            lhs_syntax: lhs.get(0).copied(),
-            rhs_syntax: rhs.get(0).copied(),
-        };
+        let start = Vertex::new(lhs.get(0).copied(), rhs.get(0).copied());
         let route = shortest_path(start);
 
         let actions = route.iter().map(|(action, _)| *action).collect_vec();
@@ -505,10 +477,7 @@ mod tests {
         )];
         init_all_info(&lhs, &rhs);
 
-        let start = Vertex {
-            lhs_syntax: lhs.get(0).copied(),
-            rhs_syntax: rhs.get(0).copied(),
-        };
+        let start = Vertex::new(lhs.get(0).copied(), rhs.get(0).copied());
         let route = shortest_path(start);
 
         let actions = route.iter().map(|(action, _)| *action).collect_vec();
@@ -539,10 +508,7 @@ mod tests {
         )];
         init_all_info(&lhs, &rhs);
 
-        let start = Vertex {
-            lhs_syntax: lhs.get(0).copied(),
-            rhs_syntax: rhs.get(0).copied(),
-        };
+        let start = Vertex::new(lhs.get(0).copied(), rhs.get(0).copied());
         let route = shortest_path(start);
 
         let actions = route.iter().map(|(action, _)| *action).collect_vec();
@@ -581,10 +547,7 @@ mod tests {
         )];
         init_all_info(&lhs, &rhs);
 
-        let start = Vertex {
-            lhs_syntax: lhs.get(0).copied(),
-            rhs_syntax: rhs.get(0).copied(),
-        };
+        let start = Vertex::new(lhs.get(0).copied(), rhs.get(0).copied());
         let route = shortest_path(start);
 
         let actions = route.iter().map(|(action, _)| *action).collect_vec();
