@@ -70,6 +70,9 @@ fn highlight_missing_style_bug(s: &str) -> String {
     s.on_purple().to_string()
 }
 
+/// Split `line` (from the source code) into multiple lines of
+/// `max_len` (i.e. word wrapping), and apply `styles` to each part
+/// according to its original position in `line`.
 pub fn split_and_apply(
     line: &str,
     max_len: usize,
@@ -91,47 +94,55 @@ pub fn split_and_apply(
     }
 
     let mut styled_parts = vec![];
-    let mut prev_length = 0;
+    let mut part_start = 0;
 
     for part in split_string(line, max_len) {
         let mut res = String::with_capacity(part.len());
-        let mut i = 0;
+        let mut prev_style_end = 0;
         for (span, style) in styles {
             // The remaining spans are beyond the end of this part.
-            if span.start_col >= prev_length + codepoint_len(&part) {
+            if span.start_col >= part_start + codepoint_len(&part) {
                 break;
             }
 
-            if i >= prev_length {
-                // Unstyled text before the next span.
-                if i < span.start_col {
-                    res.push_str(substring_by_codepoint(
-                        &part,
-                        i - prev_length,
-                        span.start_col - prev_length,
-                    ));
-                }
+            // If there's an unstyled gap before the next span.
+            if span.start_col > part_start && prev_style_end < span.start_col {
+                // Then append that text without styling.
+                let unstyled_start = max(prev_style_end, part_start);
+                res.push_str(substring_by_codepoint(
+                    &part,
+                    unstyled_start - part_start,
+                    span.start_col - part_start,
+                ));
             }
 
             // Apply style to the substring in this span.
-            if span.end_col > prev_length {
+            if span.end_col > part_start {
                 let span_s = substring_by_codepoint(
                     &part,
-                    max(0, span.start_col as isize - prev_length as isize) as usize,
-                    min(codepoint_len(&part), span.end_col - prev_length),
+                    max(0, span.start_col as isize - part_start as isize) as usize,
+                    min(codepoint_len(&part), span.end_col - part_start),
                 );
                 res.push_str(&span_s.style(*style).to_string());
             }
-            i = span.end_col;
+            prev_style_end = span.end_col;
         }
+
+        // Ensure that prev_style_end is at least at the start of this
+        // part.
+        if prev_style_end < part_start {
+            prev_style_end = part_start;
+        }
+
         // Unstyled text after the last span.
-        if i < prev_length + codepoint_len(&part) {
-            let span_s = substring_by_codepoint(&part, i - prev_length, codepoint_len(&part));
+        if prev_style_end < part_start + codepoint_len(&part) {
+            let span_s =
+                substring_by_codepoint(&part, prev_style_end - part_start, codepoint_len(&part));
             res.push_str(span_s);
         }
 
         styled_parts.push(res);
-        prev_length += codepoint_len(&part)
+        part_start += codepoint_len(&part)
     }
 
     styled_parts
@@ -229,39 +240,40 @@ pub fn color_positions(
     for pos in positions {
         let mut style = Style::new();
         match pos.kind {
-            MatchKind::UnchangedToken { highlight, .. } => match highlight {
-                TokenKind::Atom(atom_kind) => match atom_kind {
-                    AtomKind::String => {
-                        style = if background.is_dark() {
-                            style.bright_magenta()
-                        } else {
-                            style.magenta()
-                        };
+            MatchKind::UnchangedToken { highlight, .. } => {
+                if let TokenKind::Atom(atom_kind) = highlight {
+                    match atom_kind {
+                        AtomKind::String => {
+                            style = if background.is_dark() {
+                                style.bright_magenta()
+                            } else {
+                                style.magenta()
+                            };
+                        }
+                        AtomKind::Comment => {
+                            style = style.italic();
+                            style = if background.is_dark() {
+                                style.bright_blue()
+                            } else {
+                                style.blue()
+                            };
+                        }
+                        AtomKind::Keyword | AtomKind::Type => {
+                            style = style.bold();
+                        }
+                        _ => {}
                     }
-                    AtomKind::Comment => {
-                        style = style.italic();
-                        style = if background.is_dark() {
-                            style.bright_blue()
-                        } else {
-                            style.blue()
-                        };
-                    }
-                    AtomKind::Keyword | AtomKind::Type => {
-                        style = style.bold();
-                    }
-                    _ => {}
-                },
-                _ => {}
-            },
+                }
+            }
             MatchKind::Novel { highlight, .. } => {
                 style = novel_style(style, is_lhs, background);
-                match highlight {
+                if matches!(
+                    highlight,
                     TokenKind::Delimiter
-                    | TokenKind::Atom(AtomKind::Keyword)
-                    | TokenKind::Atom(AtomKind::Type) => {
-                        style = style.bold();
-                    }
-                    _ => {}
+                        | TokenKind::Atom(AtomKind::Keyword)
+                        | TokenKind::Atom(AtomKind::Type)
+                ) {
+                    style = style.bold();
                 }
                 if matches!(highlight, TokenKind::Atom(AtomKind::Comment)) {
                     style = style.italic();
@@ -344,4 +356,92 @@ mod tests {
     fn split_string_unicode() {
         assert_eq!(split_string("ab📦def", 3), vec!["ab📦", "def"]);
     }
+
+    #[test]
+    fn test_split_and_apply_missing() {
+        let res = split_and_apply("foo", 3, true, &[]);
+        assert_eq!(res, vec![highlight_missing_style_bug("foo")])
+    }
+
+    #[test]
+    fn test_split_and_apply() {
+        let res = split_and_apply(
+            "foo",
+            3,
+            true,
+            &[(
+                SingleLineSpan {
+                    line: 0.into(),
+                    start_col: 0,
+                    end_col: 3,
+                },
+                Style::new(),
+            )],
+        );
+        assert_eq!(res, vec!["foo"])
+    }
+
+    #[test]
+    fn test_split_and_apply_trailing_text() {
+        let res = split_and_apply(
+            "foobar",
+            6,
+            true,
+            &[(
+                SingleLineSpan {
+                    line: 0.into(),
+                    start_col: 0,
+                    end_col: 3,
+                },
+                Style::new(),
+            )],
+        );
+        assert_eq!(res, vec!["foobar"])
+    }
+
+    #[test]
+    fn test_split_and_apply_gap_between_styles_on_wrap_boundary() {
+        let res = split_and_apply(
+            "foobar",
+            3,
+            true,
+            &[
+                (
+                    SingleLineSpan {
+                        line: 0.into(),
+                        start_col: 0,
+                        end_col: 2,
+                    },
+                    Style::new(),
+                ),
+                (
+                    SingleLineSpan {
+                        line: 0.into(),
+                        start_col: 4,
+                        end_col: 6,
+                    },
+                    Style::new(),
+                ),
+            ],
+        );
+        assert_eq!(res, vec!["foo", "bar"])
+    }
+
+    #[test]
+    fn test_split_and_apply_trailing_text_newline() {
+        let res = split_and_apply(
+            "foobar      ",
+            6,
+            true,
+            &[(
+                SingleLineSpan {
+                    line: 0.into(),
+                    start_col: 0,
+                    end_col: 3,
+                },
+                Style::new(),
+            )],
+        );
+        assert_eq!(res, vec!["foobar", "      "])
+    }
 }
